﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics;
using Infragistics.Controls.Charts;
using System.ComponentModel;
using System.IO;
using System.Data;
using System.Collections.ObjectModel;

namespace XamDataChartSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Point? _mousePosition;
        private bool _cursorOverChart;
        public SampleViewModel vm { get; set; }

        public MainWindow()
        {
            InitializeComponent();

            vm = new SampleViewModel();
            DataContext = vm;

            vm.GraphChannels.CollectionChanged += GraphChannelsCollectionChanged;
            this.Loaded += new RoutedEventHandler(MainWindow_Loaded);
        }

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            // It is not needed to add a series to see the problems described 
            vm.AddGraphChannel();
        }

        /// <summary>
        ///   When this event fires then the mouse must be over the chart so set the cursorOverChart flag and store the mouse
        ///   position for use. If there are drug items to show then display the popup, otherwise close the popup.
        /// </summary>
        /// <param name = "sender"></param>
        /// <param name = "e"></param>
        void xmDataChart_MouseMove(object sender, MouseEventArgs e)
        {
            // we moved in the chart so we must be over the chart
            _cursorOverChart = true;

            // store the mouse position
            _mousePosition = e.GetPosition((XamDataChart)sender);

            //DisplayEventPopup(sender, null);
        }

        private void XmDataChart_OnWindowRectChanged(object sender, RectChangedEventArgs e)
        {
            //xmDataChart.ResetZoom();
            vm.Zoomed = xmDataChart.ActualWindowPositionHorizontal + xmDataChart.ActualWindowScaleHorizontal < 1;
            vm.WindowPositionHorizontal = xmDataChart.ActualWindowPositionHorizontal;
            vm.WindowScaleHorizontal = xmDataChart.ActualWindowScaleHorizontal;
        }

        void GraphChannelsCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            lock (xmDataChart)
            {
                if (e.NewItems != null)
                {
                    foreach (var graphchannel in from object newitem in e.NewItems select newitem as ChannelLookup)
                    {
                        if (graphchannel == null) continue;
                        // create the y axes
                        var channelmin = graphchannel.ViperChannels.Min(c => c.MinGraphValue);
                        var channelmax = graphchannel.ViperChannels.Max(c => c.MaxGraphValue);
                        var yaxis = new NumericYAxis
                        {
                            LabelSettings = new AxisLabelSettings
                            {
                                Visibility = Visibility.Collapsed,
                                Location = AxisLabelsLocation.InsideLeft,
                                Foreground = new SolidColorBrush(Colors.White),
                                Extent = 35
                            },
                            Label = Resources["LabelDataTemplate"] as DataTemplate,
                            Style = Resources["NumericYAxisStyle"] as Style,
                            //Interval = 25,
                            Name =
                                String.Format("y_{0}",
                                              graphchannel.ChannelName),
                            Visibility = Visibility.Collapsed,
                            MinimumValue = channelmin != null ? (double)channelmin : 0.0,
                            MaximumValue = channelmax != null ? (double)channelmax : 100
                        };
                        xmDataChart.Axes.Add(yaxis);

                        // Create the x axis
                        var xaxis = new CategoryDateTimeXAxis
                        {
                            LabelSettings = new AxisLabelSettings
                            {
                                Visibility = Visibility.Visible,
                                Location = AxisLabelsLocation.InsideBottom
                            },
                            Style = Resources["CategoryXAxisStyle"] as Style,
                            DateTimeMemberPath = "TimeStamp",
                            Name =
                                String.Format("x_{0}",
                                              graphchannel.ChannelName),
                            Visibility = Visibility.Collapsed,
                            ItemsSource = graphchannel.liveData
                        };

                        // these bindings ensure that all x axes have the same range
                        xaxis.SetBinding(CategoryDateTimeXAxis.MinimumValueProperty, new Binding("MinDateTime"));
                        xaxis.SetBinding(CategoryDateTimeXAxis.MaximumValueProperty, new Binding("MaxDateTime"));
                        xmDataChart.Axes.Add(xaxis);

                        // run the series brush through the ColorContrastConverter
                        //var colorContrastConverter = new MultiBinding();
                        //colorContrastConverter.Converter =
                        //    (LiveVue.Converters.ColorContrastConverter)Resources["ColorContrastConverter"];
                        //colorContrastConverter.Bindings.Add(new Binding("PlotAreaBackground") { Source = xmDataChart });
                        //colorContrastConverter.Bindings.Add(new Binding("Color")
                        //{
                        //    Source = graphchannel.ViperChannels.First()
                        //});

                        // Create a new series
                        var series = new StepLineSeries();
                        series.SetValue(Panel.ZIndexProperty, 1);
                        series.Title = graphchannel.ChannelName;
                        series.Name = String.Format("series_{0}", graphchannel.ChannelName);
                        series.MarkerType = MarkerType.None;
                        series.ValueMemberPath = "Value";
                        //series.SetBinding(Series.BrushProperty, colorContrastConverter);
                        //series.Brush = (Brush)(new BrushConverter().ConvertFrom(channel.ViperChannels.First().Color));
                        series.XAxis = xaxis;
                        series.YAxis = yaxis;
                        series.Legend = xmLegendOTC;
                        //series.DashArray = dashArray;
                        series.ItemsSource = graphchannel.liveData;
                        xmDataChart.Series.Add(series);
                    }
                }

                if (e.OldItems != null)
                {
                    foreach (var graphchannel in from object olditem in e.OldItems select olditem as ChannelLookup)
                    {
                        if (graphchannel == null) continue;
                        var name = graphchannel.ChannelName;
                        BindingOperations.ClearAllBindings(xmDataChart.Axes[String.Format("x_{0}", name)]);
                        xmDataChart.Axes.Remove(xmDataChart.Axes[String.Format("x_{0}", name)]);
                        xmDataChart.Axes.Remove(xmDataChart.Axes[String.Format("y_{0}", name)]);
                        xmDataChart.Series.Remove(xmDataChart.Series[String.Format("series_{0}", name)]);
                    }
                }
            }
        }
    }
}
