﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Threading;
using System.Collections.ObjectModel;
using System.Data;
using System.Threading.Tasks;

namespace XamDataChartSample
{
    public class SampleViewModel : BindableObject
    {
        #region Properties
        private System.Diagnostics.Stopwatch sw = new System.Diagnostics.Stopwatch();
        private double mspertick = (double)1000 / System.Diagnostics.Stopwatch.Frequency;
        private DispatcherTimer UpdateGraphCurrentTimeTimer;

        //public Viper InstanceViper { get; set; }

        private int _viewSizeinSeconds;
        public int ViewSizeinSeconds
        {
            get { return _viewSizeinSeconds; }
            set
            {
                SetField(ref _viewSizeinSeconds, value, "ViewSizeinSeconds");
                OnPropertyChanged("XAxisInterval");
                UpdateZoomandPosition();
            }
        }

        public double XAxisInterval
        {
            get { return TimeSpan.FromSeconds((double)ViewSizeinSeconds / 5).Ticks; }
        }

        private double _windowPositionHorizontal;
        public double WindowPositionHorizontal
        {
            get { return _windowPositionHorizontal; }
            set { SetField(ref _windowPositionHorizontal, value, "WindowPositionHorizontal"); }
        }

        private double _windowScaleHorizontal;
        public double WindowScaleHorizontal
        {
            get { return _windowScaleHorizontal; }
            set { SetField(ref _windowScaleHorizontal, value, "WindowScaleHorizontal"); }
        }

        private bool _zoomed;
        public bool Zoomed
        {
            get { return _zoomed; }
            set { SetField(ref _zoomed, value, "Zoomed"); }
        }

        private DateTime _mindatetime;
        public DateTime MinDateTime
        {
            get { return _mindatetime; }
            set { SetField(ref _mindatetime, value, "MinDateTime"); }
        }

        private DateTime _maxdatetime;
        public DateTime MaxDateTime
        {
            get { return _maxdatetime; }
            set { SetField(ref _maxdatetime, value, "MaxDateTime"); }
        }

        private DataTable _eventDataTable;
        public DataTable EventDataTable
        {
            get {
                return _eventDataTable; 
            }
            set { SetField(ref _eventDataTable, value, "EventDataTable"); }
        }

        private bool _hasSelectedGauge;
        public bool HasSelectedGauge
        {
            get
            {
                return _hasSelectedGauge;
            }
            set { SetField(ref _hasSelectedGauge, value, "HasSelectedGauge"); }
        }

        public ObservableCollection<ChannelLookup> GraphChannels { get; set; }
        #endregion

        #region Constructors
        public SampleViewModel()
        {
            //this.Gauges

            ViewSizeinSeconds = 300; // initialize the queue size to 5 mins
            MaxDateTime = DateTime.Now;
            MaxDateTime = DateTime.Now.AddSeconds(MaxDateTime.Second * -1);
            MinDateTime = MaxDateTime.AddSeconds(ViewSizeinSeconds * -2);
            GraphChannels = new ObservableCollection<ChannelLookup>();

            UpdateGraphCurrentTimeTimer = new DispatcherTimer();
            UpdateGraphCurrentTimeTimer.Tick += UpdateMaxTime;
            UpdateGraphCurrentTimeTimer.Interval = new TimeSpan(0, 0, 0, 1, 0);
            UpdateGraphCurrentTimeTimer.Start();
        }
        #endregion


        public void AddGraphChannel()
        {
            // Add the selected channel to the list of channels added to the graph
            // this channel name has not been added
            var channel = new ChannelLookup
                                {
                                    ChannelName = "Pulse",
                                    Selected = true,
                                    liveData = new ObservableCollection<ChannelData>(),
                                    ViperChannels = new List<CaseChannel> { new CaseChannel
                                                                                {
                                                                                    ChannelIndex = 1, 
                                                                                    ChannelType = 2,
                                                                                    MinGraphValue = 0,
                                                                                    MaxGraphValue = 220
                                                                                }
                                    }
                                };

            GraphChannels.Add(channel);
        }

        private void UpdateMaxTime(object sender, EventArgs e)
        {
            Application.Current.Dispatcher.Invoke(
                DispatcherPriority.Normal,
                (Action) delegate
                                {
                                    MaxDateTime = DateTime.Now;
                                    UpdateZoomandPosition();
                                });
        }

        private void UpdateZoomandPosition()
        {
            // When the user is zoomed then we just added another second to the dataset which shifts it 
            // right so we need to shift left. Determine what percentage is a single second and substract
            // that from the current position
            var span = MaxDateTime.Subtract(MinDateTime).TotalSeconds;
            if (Zoomed)
            {
                // With no adjustment the graph scroll off to the left.
                // Shifting it to the left by 1/span seems to be too much. Graph scrolls to the right.
                // Depending on the position of the viewport the shifting seems to be different. If close to the right then it shifts fast. However, if close to the left it barely shifts at all.

                //WindowPositionHorizontal = WindowPositionHorizontal - (1 / (2 * span)) ;
                WindowPositionHorizontal = WindowPositionHorizontal - (1 / span); // shift the position to the left by the size of the added time
                WindowScaleHorizontal = (WindowScaleHorizontal * (span - 1)) / span; // reduce the size of the zoom to be the same proportion after the added time
            }
            else
            {
                // When the user has not zoomed or scrolled the window we need to keep the view 
                // set to the far right to see the most recent data and set the zoom so that only
                // the selected number of seconds is showing
                WindowScaleHorizontal = ViewSizeinSeconds/span;
                WindowPositionHorizontal = 1 - WindowScaleHorizontal;
            }
        }
    }

    public class ChannelLookup : BindableObject
    {
        public string ChannelName { get; set; }
        public List<CaseChannel> ViperChannels { get; set; }
        //public ObservableQueue<ChannelData> liveData;
        public ObservableCollection<ChannelData> liveData { get; set; }

        private bool _selected;
        public bool Selected
        {
            get { return _selected; }
            set { SetField(ref _selected, value, "Selected"); }
        }
    }

    public class ChannelData
    {
        public int ChannelIndex { get; set; }
        public string ChannelName { get; set; }
        public double? Value { get; set; }
        public DateTime TimeStamp { get; set; }

        public ChannelData Clone()
        {
            return (ChannelData)MemberwiseClone();
        }
    }

    public class CaseChannel
    {
        public string SerialNumber { get; set; }
        public int ChannelIndex { get; set; }
        public string MeasurementName { get; set; }
        public double? MinGraphValue { get; set; }
        public double? MaxGraphValue { get; set; }
        public string WebName { get; set; }
        public string ChannelUnits { get; set; }
        public string FormatSpecifier { get; set; }
        public int ChannelType { get; set; }
        public string Color { get; set; }
        public int? MeasurementGroupId { get; set; }

        public CaseChannel Clone()
        {
            return (CaseChannel)MemberwiseClone();
        }
    }
}
